
from distutils.core import setup
setup(
  name = 'parski',
  packages = ['parski'],
  version = '1.1.2.42',
  description = 'Helper library for parsing vpc data feed',
  author = 'Russell Endicott',
  author_email = 'rendicott@gmail.com',
  url = 'https://github.build.company.com/CloudPod/parski', # use the URL to the github repo
  download_url = 'https://github.build.company.com/CloudPod/parski/releases/download/1.1.2.42/parski-1.1.2.42.tar.gz', # I'll explain this in a second
  keywords = ['vpc', 'json', 'parski'],
  classifiers = [],
)